@@ -41,18 +41,17 @@
 #include "../src/myalloca.h"
 
 // not needed anymore
-//void *memmem (const void *big, size_t big_len, const void *little,
+// void *memmem (const void *big, size_t big_len, const void *little,
 //              size_t little_len);
 
-
 #define CHAIN_SIZE 128
 int cur_hdl; // to avoid dupl. search
 
 #include "unknown.h"
 
 static struct _unknown_dxf unknown_dxf[] = {
-  // see log_unknown_dxf.pl
-  #include "alldxf_0.inc"
+// see log_unknown_dxf.pl
+#include "alldxf_0.inc"
   { NULL, NULL, 0, "", 0, 0, 0, 0, 0, 0, 0, NULL }
 };
 
@@ -100,8 +99,8 @@ static struct _bd
   const char *value;
   const char *bin;
 } bd[] = {
-  // see bd-unknown.pl
-  #include "bd-unknown.inc"
+// see bd-unknown.pl
+#include "bd-unknown.inc"
   { NULL, NULL }
 };
 
@@ -116,7 +115,7 @@ static void
 bits_TU (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
 {
   // the source is always utf8
-  BITCODE_TU wstr = bit_utf8_to_TU ((char*)g->value, 0);
+  BITCODE_TU wstr = bit_utf8_to_TU ((char *)g->value, 0);
   bit_write_TU (dat, wstr);
   g->type = BITS_TU;
   free (wstr);
@@ -190,13 +189,13 @@ bits_BD (Bit_Chain *restrict dat, struct _unknown_field *restrict g)
   // 4.0, 2.0, 0.36, 5.0, 4.131293495034893, 4701.847034571434,
   // 0.5024999976158142
   // hard-code some special values not properly converted and found.
-  if (!strcmp (g->value, "0.5"))
+  if (strEQc (g->value, "0.5"))
     d = 0.5;
-  else if (!strcmp (g->value, "10.0"))
+  else if (strEQc (g->value, "10.0"))
     d = 10.0;
-  else if (!strcmp (g->value, "11.0"))
+  else if (strEQc (g->value, "11.0"))
     d = 11.0;
-  else if (!strcmp (g->value, "63.5"))
+  else if (strEQc (g->value, "63.5"))
     d = 63.5;
   g->type = BITS_BD;
   // some more not found BD values
@@ -582,7 +581,7 @@ num_dxf (const struct _unknown_field *g, const struct _unknown_dxf *dxf) {
   int num_dxf = 0;
   struct _unknown_field *f = (struct _unknown_field *)dxf->fields;
   while (f->value) {
-    if (f->code == g->code && !strcmp(f->value, g->value))
+    if (f->code == g->code && strEQ (f->value, g->value))
       num_dxf++;
     f++;
   }
@@ -822,11 +821,11 @@ close_pi (FILE *pi, long class_filled, long class_size, int k)
 int
 main (int argc, char *argv[])
 {
-  int i = 1, ic, j, num_classes;
-  long sum_filled = 0, sum_size = 0;
+  unsigned i = 1, ic, j, num_classes = 0;
+  unsigned long sum_filled = 0, sum_size = 0;
   char *class = NULL;
   char *file = NULL;
-#define MAX_CLASSES 100
+#define MAX_CLASSES 200
   char *classes[MAX_CLASSES]; // create files per classes
   struct _dxf *dxf = calloc (sizeof (unknown_dxf) / sizeof (unknown_dxf[0]),
                              sizeof (struct _dxf));
@@ -834,12 +833,12 @@ main (int argc, char *argv[])
   #include "alldxf_2.inc"
   // clang-format on
 
-  if (argc > 2 && !strcmp (argv[i], "--class"))
+  if (argc > 2 && strEQc (argv[i], "--class"))
     {
       class = argv[i + 1];
       i = 3;
     }
-  if (argc - i >= 2 && !strcmp (argv[i], "--file"))
+  if (argc - i >= 2 && strEQc (argv[i], "--file"))
     file = argv[i + 1];
   // process per class, not per logged instances.
   if (!class)
@@ -847,7 +846,7 @@ main (int argc, char *argv[])
       num_classes = 0;
       for (i = 0; unknown_dxf[i].name; i++)
         { // TODO: alldwg/alldxf needs to be sorted per class, not file.
-          if (class != unknown_dxf[i].name)
+          if (!class || !strEQ (class, unknown_dxf[i].name))
             {
               classes[num_classes++] = (char *)unknown_dxf[i].name;
               class = (char *)unknown_dxf[i].name;
@@ -876,31 +875,37 @@ main (int argc, char *argv[])
       uint16_t i_pi = 0;
 
       class = classes[ic];
-      // dirname should be examples/
+      // dirname should be examples, but can also be examples/.libs
       dn = dirname (argv[0]);
       printf ("dirname(%s): %s\n", argv[0], dn);
-      if (dn && !strcmp (basename (dn), "examples"))
+      if (dn && strstr (dn, "/examples/.libs"))
+        {
+          size_t l = strlen (dn);
+          strcpy (pi_fn, dn);
+          pi_fn[l - 5] = '\0';
+        }
+      else if (dn && strEQc (basename (dn), "examples"))
         {
           strcpy (pi_fn, dn);
           strcat (pi_fn, "/");
         }
-      else if (!strcmp (argv[0], "examples"))
+      else if (strEQc (argv[0], "examples"))
         {
           strcpy (pi_fn, "examples/");
         }
       else
-          strcpy (pi_fn, "");
+        strcpy (pi_fn, "");
       strcat (pi_fn, class);
       strcpy (base_pi_fn, pi_fn);
       pi_filename (pi_fn, i_pi);
       pi = fopen (pi_fn, "w");
       if (open_pi (pi, class, pi_fn))
-          continue;
+        continue;
       for (i = 0; unknown_dxf[i].name; i++)
         {
-          int num_fields;
-          int num_found = -1;
-          int size = unknown_dxf[i].num_bits;
+          unsigned num_fields;
+          unsigned num_found = -1;
+          unsigned size = unknown_dxf[i].num_bits;
           struct _unknown_field *g
               = (struct _unknown_field *)unknown_dxf[i].fields;
           const int is16 = dxf_is16 (&unknown_dxf[i]);
@@ -910,11 +915,11 @@ main (int argc, char *argv[])
           int version = 0;
           char *s;
 
-          if (class && strcmp (class, unknown_dxf[i].name))
+          if (class && !strEQ (class, unknown_dxf[i].name))
             continue;
-          if (file && strcmp (file, unknown_dxf[i].dxf))
+          if (file && !strEQ (file, unknown_dxf[i].dxf))
             continue;
-          /*if (!strcmp(unknown_dxf[i].name, "TABLEGEOMETRY")) {
+          /*if (strEQc (unknown_dxf[i].name, "TABLEGEOMETRY")) {
             printf("skip TABLEGEOMETRY\n");
             continue;
           }*/
@@ -960,11 +965,11 @@ main (int argc, char *argv[])
               printf ("%d: %s\n", g[j].code, g[j].value);
               if (g[j].code == 102)
                 {
-                  if (!strcmp (g[j].value, "{ACAD_XDICTIONARY"))
+                  if (strEQc (g[j].value, "{ACAD_XDICTIONARY"))
                     is_dict = 1;
-                  else if (!strcmp (g[j].value, "{ACAD_REACTORS"))
+                  else if (strEQc (g[j].value, "{ACAD_REACTORS"))
                     is_react = 1;
-                  else if (!strcmp (g[j].value, "}"))
+                  else if (strEQc (g[j].value, "}"))
                     {
                       is_react = 0;
                       is_dict = 0;
@@ -1172,7 +1177,8 @@ main (int argc, char *argv[])
                           if (num_found)
                             {
                               free (dat.chain);
-                              dat.chain = (unsigned char *)unknown_dxf[i].bytes;
+                              dat.chain
+                                  = (unsigned char *)unknown_dxf[i].bytes;
                               g[j].bytes = dat.chain;
                               dat.size = unknown_dxf[i].num_bits / 8;
                               bit_set_position (&dat, g[j].pos[0]);
@@ -1241,7 +1247,8 @@ main (int argc, char *argv[])
                           if (num_found)
                             {
                               free (dat.chain);
-                              dat.chain = (unsigned char *)unknown_dxf[i].bytes;
+                              dat.chain
+                                  = (unsigned char *)unknown_dxf[i].bytes;
                               dat.size = unknown_dxf[i].num_bits / 8;
                               bit_set_position (&dat, g[j].pos[0]);
                               d = bit_read_BD (&dat);
@@ -1302,7 +1309,7 @@ main (int argc, char *argv[])
                   if (!num_found || is_common_entity_data (g[j].code))
                     {
                       piname = (char *)dwg_bits_name[g[j].type];
-                      if (!strcmp (piname, "HANDLE"))
+                      if (strEQc (piname, "HANDLE"))
                         piname = (char *)"H";
                       // unfound DXF field for the picat file.
                       // later could be used as hints for the picat solver
@@ -1512,10 +1519,11 @@ main (int argc, char *argv[])
                 &g[j])) { offset = g[j].pos[0]+1; goto SEARCH;
                 }
                 */
-              if (0) {
-                free (g[j].bytes);
-                g[j].bytes = NULL;
-              }
+              if (0)
+                {
+                  free (g[j].bytes);
+                  g[j].bytes = NULL;
+                }
             }
           fprintf (pi, "  ],\n"
                        "  Data = [S,Fields,Class,Dxf,Version,Offsets],\n"
@@ -1566,8 +1574,8 @@ main (int argc, char *argv[])
 
           // TODO: try likely field combinations and print the top 3.
           // See unknown.pi
-          // there are various heuristics, like the handle and string stream at the end.
-          // points BD's being neighbors, ...
+          // there are various heuristics, like the handle and string stream at
+          // the end. points BD's being neighbors, ...
 
           free (dxf[i].found);
           free (dxf[i].possible);
