@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2023 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -45,7 +45,7 @@ static int env_var_checked_p;
 
 /* the current version per spec block */
 static unsigned int cur_ver = 0;
-static Bit_Chain pdat = { NULL, 0, 0, 0, 0, R_INVALID };
+static Bit_Chain pdat = { 0 };
 static BITCODE_BL rcount1, rcount2;
 
 /*--------------------------------------------------------------------------------
@@ -67,21 +67,18 @@ static BITCODE_BL rcount1, rcount2;
 #undef PRE
 #undef VERSIONS
 #undef VERSION
-#define UNTIL(v)                                                              \
-  if (dat->from_version <= v)
-#define SINCE(v)                                                              \
-  if (dat->from_version >= v)
-#define PRE(v)                                                                \
-  if (dat->from_version < v)
+#define UNTIL(v) if (dat->from_version <= v)
+#define SINCE(v) if (dat->from_version >= v)
+#define PRE(v) if (dat->from_version < v)
 #define VERSIONS(v1, v2)                                                      \
   if (dat->from_version >= v1 && dat->from_version <= v2)
-#define VERSION(v)                                                            \
-  if (dat->from_version == v)
+#define VERSION(v) if (dat->from_version == v)
 
 #define VALUE(value, type, dxf)
 #define VALUE_RC(value, dxf) VALUE (value, RC, dxf)
 #define VALUE_RS(value, dxf) VALUE (value, RS, dxf)
 #define VALUE_RL(value, dxf) VALUE (value, RL, dxf)
+#define VALUE_RLx(value, dxf) VALUE (value, RL, dxf)
 #define VALUE_RD(value, dxf) VALUE (value, RD, dxf)
 #define VALUE_BD(value, dxf) VALUE (value, BD, dxf)
 
@@ -105,6 +102,8 @@ static BITCODE_BL rcount1, rcount2;
 #define ANYCODE -1
 #define FIELD_HANDLE(nam, code, dxf) VALUE_HANDLE (_obj->nam, nam, code, dxf)
 #define SUB_FIELD_HANDLE(o, nam, code, dxf)                                   \
+  if (obj->fixedtype == DWG_TYPE_TABLESTYLE)                                  \
+    LOG_HANDLE ("free TABLESTYLE.%s.%s\n", #o, #nam);                         \
   VALUE_HANDLE (_obj->o.nam, nam, code, dxf)
 // compare to dwg_decode_handleref_with_code: not all refs are stored in the
 // object_ref vector, like relative ptrs and NULL.
@@ -118,6 +117,8 @@ static BITCODE_BL rcount1, rcount2;
     } /* else freed globally */
 #define FIELD_DATAHANDLE(name, code, dxf) FIELD_HANDLE (name, code, dxf)
 #define FIELD_HANDLE_N(name, vcount, code, dxf) FIELD_HANDLE (name, code, dxf)
+#define FIELD_VECTOR_INL(nam, type, size, dxf)
+#define VALUE_H(hdl, dxf)
 
 #define FIELD_B(name, dxf) FIELD (name, B)
 #define FIELD_BB(name, dxf) FIELD (name, BB)
@@ -139,6 +140,7 @@ static BITCODE_BL rcount1, rcount2;
 #define VALUE_TFF(value, dxf)
 #define FIELD_TU(name, dxf) FIELD_TV (name, dxf)
 #define FIELD_TF(name, len, dxf) FIELD_TV (name, dxf)
+#define FIELD_TFv(name, len, dxf) FIELD_TV (name, dxf)
 #define FIELD_TFF(name, len, dxf)                                             \
   {                                                                           \
   }
@@ -193,11 +195,13 @@ static BITCODE_BL rcount1, rcount2;
 #define FIELD_TIMEBLL(name, dxf)
 #define FIELD_TIMERLL(name, dxf)
 #define FIELD_CMC(color, dxf)                                                 \
+  SINCE (R_2004a)                                                              \
   {                                                                           \
     FIELD_T (color.name, 0);                                                  \
     FIELD_T (color.book_name, 0);                                             \
   }
 #define SUB_FIELD_CMC(o, color, dxf)                                          \
+  SINCE (R_2004a)                                                              \
   {                                                                           \
     VALUE_TV (_obj->o.color.name, 0);                                         \
     VALUE_TV (_obj->o.color.book_name, 0);                                    \
@@ -209,7 +213,7 @@ static BITCODE_BL rcount1, rcount2;
 #define FIELD_VECTOR_N(nam, type, size, dxf)                                  \
   if ((size) && _obj->nam)                                                    \
     {                                                                         \
-      for (vcount = 0; vcount < (BITCODE_BL) (size); vcount++)                \
+      for (vcount = 0; vcount < (BITCODE_BL)(size); vcount++)                 \
         FIELD_##type (nam[vcount], dxf);                                      \
     }                                                                         \
   FIELD_TV (nam, dxf);
@@ -227,26 +231,23 @@ static BITCODE_BL rcount1, rcount2;
         {                                                                     \
           FIELD_HANDLE_N (name[vcount], vcount, code, dxf);                   \
         }                                                                     \
-      if (size)                                                               \
-        {                                                                     \
-          FIELD_TV (name, dxf)                                                \
-        }                                                                     \
+      FIELD_TV (name, dxf)                                                    \
     }
 #define HANDLE_VECTOR(name, sizefield, code, dxf)                             \
   HANDLE_VECTOR_N (name, FIELD_VALUE (sizefield), code, dxf)
-#define SUB_HANDLE_VECTOR(o,name, sizefield, code, dxf)                       \
+#define SUB_HANDLE_VECTOR(o, name, sizefield, code, dxf)                      \
   if (_obj->o.name && _obj->o.sizefield)                                      \
     {                                                                         \
       for (vcount = 0; vcount < (BITCODE_BL)_obj->o.sizefield; vcount++)      \
         {                                                                     \
-          SUB_FIELD_HANDLE (o,name[vcount], code, dxf);                       \
+          SUB_FIELD_HANDLE (o, name[vcount], code, dxf);                      \
         }                                                                     \
       FREE_IF (_obj->o.name)                                                  \
     }
-#define SUB_FIELD_VECTOR(o, nam, sizefield, type, dxf)                        \
+#define SUB_FIELD_VECTOR(o, nam, type, sizefield, dxf)                        \
   if (_obj->o.sizefield && _obj->o.nam)                                       \
     {                                                                         \
-      for (vcount = 0; vcount < (BITCODE_BL) (_obj->o.sizefield); vcount++)   \
+      for (vcount = 0; vcount < (BITCODE_BL)(_obj->o.sizefield); vcount++)    \
         SUB_FIELD_##type (o, nam[vcount], dxf);                               \
     }                                                                         \
   FREE_IF (_obj->o.nam)
@@ -263,16 +264,16 @@ static BITCODE_BL rcount1, rcount2;
       VALUE_TV (obj->tio.object->reactors, 0);                                \
     }
 #define ENT_REACTORS(code)                                                    \
-  if (ent->reactors)                                                          \
+  if (_ent->reactors)                                                         \
     {                                                                         \
-      for (vcount = 0; vcount < ent->num_reactors; vcount++)                  \
-        VALUE_HANDLE (ent->reactors[vcount], reactors, code, 330);            \
-      VALUE_TV (ent->reactors, 0);                                            \
+      for (vcount = 0; vcount < _ent->num_reactors; vcount++)                 \
+        VALUE_HANDLE (_ent->reactors[vcount], reactors, code, 330);           \
+      VALUE_TV (_ent->reactors, 0);                                           \
     }
 #define XDICOBJHANDLE(code)                                                   \
-  SINCE (R_2004)                                                              \
+  SINCE (R_2004a)                                                              \
   {                                                                           \
-    if (!obj->tio.object->is_xdic_missing)                                  \
+    if (!obj->tio.object->is_xdic_missing)                                    \
       {                                                                       \
         VALUE_HANDLE (obj->tio.object->xdicobjhandle, xdicobjhandle, code,    \
                       0);                                                     \
@@ -283,14 +284,17 @@ static BITCODE_BL rcount1, rcount2;
     VALUE_HANDLE (obj->tio.object->xdicobjhandle, xdicobjhandle, code, 0);    \
   }
 #define ENT_XDICOBJHANDLE(code)                                               \
-  SINCE (R_2004)                                                              \
+  SINCE (R_2004a)                                                              \
   {                                                                           \
-    if (!ent->is_xdic_missing)                                              \
+    if (!_ent->is_xdic_missing)                                               \
       {                                                                       \
-        VALUE_HANDLE (ent->xdicobjhandle, xdicobjhandle, code, 0);            \
+        VALUE_HANDLE (_ent->xdicobjhandle, xdicobjhandle, code, 0);           \
       }                                                                       \
   }                                                                           \
-  PRIOR_VERSIONS { VALUE_HANDLE (ent->xdicobjhandle, xdicobjhandle, code, 0); }
+  PRIOR_VERSIONS                                                              \
+  {                                                                           \
+    VALUE_HANDLE (_ent->xdicobjhandle, xdicobjhandle, code, 0);               \
+  }
 
 #define END_REPEAT(field) FIELD_TV (field, 0)
 
@@ -300,11 +304,12 @@ static BITCODE_BL rcount1, rcount2;
 #define END_STRING_STREAM
 #define START_HANDLE_STREAM
 
+#include "spec.h"
 
 #define DWG_ENTITY(token)                                                     \
-  static int dwg_free_##token##_private                                       \
-    (Bit_Chain *dat, Bit_Chain *hdl_dat,                                      \
-     Bit_Chain *str_dat, Dwg_Object *restrict obj);                           \
+  static int dwg_free_##token##_private (Bit_Chain *dat, Bit_Chain *hdl_dat,  \
+                                         Bit_Chain *str_dat,                  \
+                                         Dwg_Object *restrict obj);           \
                                                                               \
   static int dwg_free_##token (Bit_Chain *restrict dat,                       \
                                Dwg_Object *restrict obj)                      \
@@ -327,9 +332,9 @@ static BITCODE_BL rcount1, rcount2;
     obj->parent = NULL;                                                       \
     return error;                                                             \
   }                                                                           \
-  static int dwg_free_##token##_private                                       \
-    (Bit_Chain *dat, Bit_Chain *hdl_dat,                                      \
-     Bit_Chain *str_dat, Dwg_Object *restrict obj)                            \
+  static int dwg_free_##token##_private (Bit_Chain *dat, Bit_Chain *hdl_dat,  \
+                                         Bit_Chain *str_dat,                  \
+                                         Dwg_Object *restrict obj)            \
   {                                                                           \
     BITCODE_BL vcount, rcount3, rcount4;                                      \
     Dwg_Entity_##token *ent, *_obj;                                           \
@@ -342,13 +347,14 @@ static BITCODE_BL rcount1, rcount2;
     _obj = ent = _ent->tio.token;
 
 #define DWG_ENTITY_END                                                        \
-    return error;                                                             \
+    FREE_IF (obj->unknown_rest);                                              \
+  return error;                                                               \
   }
 
 #define DWG_OBJECT(token)                                                     \
-  static int dwg_free_##token##_private                                       \
-    (Bit_Chain *dat, Bit_Chain *hdl_dat,                                      \
-     Bit_Chain *str_dat, Dwg_Object *restrict obj);                           \
+  static int dwg_free_##token##_private (Bit_Chain *dat, Bit_Chain *hdl_dat,  \
+                                         Bit_Chain *str_dat,                  \
+                                         Dwg_Object *restrict obj);           \
                                                                               \
   static int dwg_free_##token (Bit_Chain *restrict dat,                       \
                                Dwg_Object *restrict obj)                      \
@@ -369,9 +375,9 @@ static BITCODE_BL rcount1, rcount2;
     return error;                                                             \
   }                                                                           \
                                                                               \
-  static int dwg_free_##token##_private                                       \
-    (Bit_Chain *dat, Bit_Chain *hdl_dat,                                      \
-     Bit_Chain *str_dat, Dwg_Object *restrict obj)                            \
+  static int dwg_free_##token##_private (Bit_Chain *dat, Bit_Chain *hdl_dat,  \
+                                         Bit_Chain *str_dat,                  \
+                                         Dwg_Object *restrict obj)            \
   {                                                                           \
     BITCODE_BL vcount, rcount3, rcount4;                                      \
     Dwg_Object_##token *_obj;                                                 \
@@ -384,7 +390,8 @@ static BITCODE_BL rcount1, rcount2;
 /* obj itself is allocated via dwg->object[], dxfname is klass->dxfname or
  * static */
 #define DWG_OBJECT_END                                                        \
-    return error;                                                             \
+    FREE_IF (obj->unknown_rest);                                              \
+  return error;                                                               \
   }
 
 static void
@@ -395,23 +402,23 @@ dwg_free_common_entity_data (Dwg_Object *obj)
   Bit_Chain *dat = &pdat;
   Bit_Chain *hdl_dat = &pdat;
   Dwg_Object_Entity *_obj;
-  Dwg_Object_Entity *ent;
+  Dwg_Object_Entity *_ent;
   BITCODE_BL vcount;
   int error = 0;
 
-  ent = obj->tio.entity;
-  if (!ent)
+  _ent = obj->tio.entity;
+  if (!_ent)
     return;
-  _obj = ent;
+  _obj = _ent;
 
-  FREE_IF (ent->preview);
+  FREE_IF (_ent->preview);
 
-  // clang-format off
+// clang-format off
   #include "common_entity_data.spec"
-  if (dat->from_version >= R_2007 && ent->color.flag & 0x40)
+  if (dat->from_version >= R_2007 && _ent->color.flag & 0x40)
     FIELD_HANDLE (color.handle, 0, 430);
-  SINCE (R_13) {
-  #include "common_entity_handle_data.spec"
+  SINCE (R_13b1) {
+    #include "common_entity_handle_data.spec"
   }
   // clang-format on
 }
@@ -427,7 +434,7 @@ dwg_free_common_object_data (Dwg_Object *obj)
   BITCODE_BL vcount;
   int error = 0;
 
-  // clang-format off
+// clang-format off
   #include "common_object_handle_data.spec"
   // clang-format on
 }
@@ -472,7 +479,8 @@ dwg_free_eed (Dwg_Object *obj)
 #include "dwg.spec"
 
 // could be a hash or switch, but there are not that many DEBUGGING classes.
-// but a switch is fine, as we get all missing types in objects.inc, generated by regen-dynapi
+// but a switch is fine, as we get all missing types in objects.inc, generated
+// by regen-dynapi
 int
 dwg_free_variable_no_class (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
 {
@@ -480,22 +488,26 @@ dwg_free_variable_no_class (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
 
 #undef DWG_ENTITY
 #undef DWG_OBJECT
-#define FREE_NOCLASS(name) case DWG_TYPE_##name: return dwg_free_##name (dat, obj);
+#define FREE_NOCLASS(name)                                                    \
+  case DWG_TYPE_##name:                                                       \
+    return dwg_free_##name (dat, obj);
 #define DWG_ENTITY(name) FREE_NOCLASS (name)
 #define DWG_OBJECT(name) FREE_NOCLASS (name)
 
   switch (obj->fixedtype)
     {
-    #include "objects.inc"
+#include "objects.inc"
+
+    case DWG_TYPE_FREED:
+      break; // already freed
 
-    case DWG_TYPE_FREED: break; // already freed
-    case DWG_TYPE_UNUSED:
     case DWG_TYPE_ACDSRECORD:
     case DWG_TYPE_ACDSSCHEMA:
     case DWG_TYPE_NPOCOLLECTION:
     case DWG_TYPE_XREFPANELOBJECT:
-    default: LOG_ERROR ("Unhandled class %s, fixedtype %d in objects.inc",
-                        dwg_type_name (obj->fixedtype), (int)obj->fixedtype);
+    default:
+      LOG_ERROR ("Unhandled class %s, fixedtype %d in objects.inc",
+                 dwg_type_name (obj->fixedtype), (int)obj->fixedtype);
     }
 
 #undef DWG_ENTITY
@@ -507,7 +519,8 @@ dwg_free_variable_no_class (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
 
 /* returns error.
  */
-int dwg_free_variable_type (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
+int
+dwg_free_variable_type (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
 {
   const int i = obj->type - 500;
   Dwg_Class *klass;
@@ -521,7 +534,7 @@ int dwg_free_variable_type (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
     }
 
   klass = &dwg->dwg_class[i];
-  if (!klass || !klass->dxfname)
+  if (!klass || !klass->dxfname || !obj->dxfname)
     {
       LOG_WARN ("No class for %s type %d found", obj->name, obj->type);
       return dwg_free_variable_no_class (dwg, obj);
@@ -535,17 +548,17 @@ int dwg_free_variable_type (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
         return DWG_ERR_UNHANDLEDCLASS;
       else
         {
-          LOG_ERROR ("Wrong %s.type %d for obj [%d]: != %s",  obj->dxfname, obj->type,
-                     obj->index, klass->dxfname);
+          LOG_ERROR ("Wrong %s.type %d for obj [%d]: != %s", obj->dxfname,
+                     obj->type, obj->index, klass->dxfname);
           return dwg_free_variable_no_class (dwg, obj);
         }
     }
 
-  // global class dispatcher:
-  // with indxf even DEBUGGING objects, such as TABLE are created.
-  // usually not written/encoded though.
+    // global class dispatcher:
+    // with indxf even DEBUGGING objects, such as TABLE are created.
+    // usually not written/encoded though.
 
-  // clang-format off
+    // clang-format off
   #include "classes.inc"
 
   #undef WARN_UNHANDLED_CLASS
@@ -557,29 +570,34 @@ int dwg_free_variable_type (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
 }
 
 /* returns error */
-int dwg_free_variable_type_private (Dwg_Object *restrict obj)
+int
+dwg_free_variable_type_private (Dwg_Object *restrict obj)
 {
   Dwg_Data *restrict dwg = obj->parent;
   Bit_Chain *dat = &pdat;
 
 #undef DWG_ENTITY
 #undef DWG_OBJECT
-#define FREE_NOCLASS(name) case DWG_TYPE_##name: return dwg_free_##name##_private (dat, dat, dat, obj);
+#define FREE_NOCLASS(name)                                                    \
+  case DWG_TYPE_##name:                                                       \
+    return dwg_free_##name##_private (dat, dat, dat, obj);
 #define DWG_ENTITY(name) FREE_NOCLASS (name)
 #define DWG_OBJECT(name) FREE_NOCLASS (name)
 
   switch (obj->fixedtype)
     {
-    #include "objects.inc"
+#include "objects.inc"
+
+    case DWG_TYPE_FREED:
+      break; // already freed
 
-    case DWG_TYPE_FREED: break; // already freed
-    case DWG_TYPE_UNUSED:
     case DWG_TYPE_ACDSRECORD:
     case DWG_TYPE_ACDSSCHEMA:
     case DWG_TYPE_NPOCOLLECTION:
     case DWG_TYPE_XREFPANELOBJECT:
-    default: LOG_ERROR ("Unhandled class %s, fixedtype %d in objects.inc",
-                        dwg_type_name (obj->fixedtype), (int)obj->fixedtype);
+    default:
+      LOG_ERROR ("Unhandled class %s, fixedtype %d in objects.inc",
+                 dwg_type_name (obj->fixedtype), (int)obj->fixedtype);
     }
 
 #undef DWG_ENTITY
@@ -589,6 +607,344 @@ int dwg_free_variable_type_private (Dwg_Object *restrict obj)
   return DWG_ERR_UNHANDLEDCLASS;
 }
 
+// after downconvert_TABLESTYLE()
+// we need to pass through the old code, as the new code is handled in the spec
+// free() following from_version, not version.
+static void
+free_TABLESTYLE_r2010 (Bit_Chain *restrict dat, Dwg_Object *restrict obj)
+{
+  Dwg_Object_TABLESTYLE *_obj
+      = obj->tio.object ? obj->tio.object->tio.TABLESTYLE : NULL;
+  if (!obj || obj->fixedtype != DWG_TYPE_TABLESTYLE)
+    {
+      LOG_ERROR ("Invalid fixedtype %u for free_TABLESTYLE_r2010",
+                 obj ? obj->fixedtype : 0);
+      return;
+    }
+  LOG_HANDLE ("free_TABLESTYLE_r2010\n");
+  if (_obj->rowstyles)
+    for (unsigned i = 0; i < 3; i++)
+      {
+        for (unsigned j = 0; j < 6; j++)
+          {
+            SUB_FIELD_CMTC (rowstyles[i].borders[j], color, 0);
+          }
+        FREE_IF (_obj->rowstyles[i].borders);
+        SUB_FIELD_HANDLE (rowstyles[i], text_style, 5, 7);
+        SUB_FIELD_CMTC (rowstyles[i], text_color, 0);
+        SUB_FIELD_CMTC (rowstyles[i], fill_color, 0);
+      }
+  FREE_IF (_obj->rowstyles);
+  _obj->num_rowstyles = 0;
+  for (unsigned j = 0; j < _obj->sty.cellstyle.num_borders; j++)
+    {
+      SUB_FIELD_HANDLE (sty.cellstyle.borders[j], ltype, 3, 340);
+      SUB_FIELD_CMTC (sty.cellstyle.borders[j], color, 0);
+    }
+  FIELD_CMTC (sty.cellstyle.bg_color, 62);
+  FIELD_T (sty.cellstyle.content_format.value_format_string, 300);
+  if (_obj->numoverrides)
+    {
+      for (unsigned j = 0; j < _obj->ovr.cellstyle.num_borders; j++)
+        {
+          SUB_FIELD_HANDLE (ovr.cellstyle.borders[j], ltype, 3, 340);
+          SUB_FIELD_CMTC (ovr.cellstyle.borders[j], color, 0);
+        }
+      FIELD_CMTC (ovr.cellstyle.bg_color, 62);
+      FIELD_T (ovr.cellstyle.content_format.value_format_string, 300);
+    }
+  FIELD_TV (name, 3);
+  FIELD_TV (sty.name, 300);
+  FIELD_TV (ovr.name, 300);
+}
+
+static void
+free_preR13_object (Dwg_Object *obj)
+{
+  int error = 0;
+  long unsigned int j;
+  Dwg_Data *dwg;
+  Bit_Chain *dat = &pdat;
+
+  // if (obj->name)
+  //   LOG_HANDLE ("free_preR13_object: %s %d\n", obj->name, obj->index)
+  if (obj && obj->parent)
+    {
+      dwg = obj->parent;
+      dat->version = dwg->header.version;
+      dat->from_version = dwg->header.from_version;
+    }
+  else
+    return;
+  if (obj->type == DWG_TYPE_FREED || obj->tio.object == NULL)
+    return;
+
+  if (obj->supertype == DWG_SUPERTYPE_ENTITY)
+    {
+      Dwg_Object_Entity *_obj = obj->tio.entity;
+      FIELD_HANDLE (layer, 2, 8);
+      if (_obj->flag_r11 & FLAG_R11_HAS_LTYPE) // 2
+        FIELD_HANDLE (ltype, 1, 6);
+      if (_obj->flag_r11 & FLAG_R11_HAS_HANDLING)
+        {   // 32
+          ; // obj->handle is static
+        }
+    }
+
+  if (obj->fixedtype == DWG_TYPE_UNUSED // deleted
+      && dwg->header.version < R_2_0b && obj->type > 64)
+    {
+      obj->type = -(int8_t)obj->type;
+      // handle only entities with extra vectors specially
+      switch (obj->type)
+        {
+        case DWG_TYPE_TEXT_r11:
+          dwg_free_TEXT (dat, obj);
+          break;
+        case DWG_TYPE_ATTRIB_r11:
+          dwg_free_ATTRIB (dat, obj);
+          break;
+        case DWG_TYPE_ATTDEF_r11:
+          dwg_free_ATTDEF (dat, obj);
+          break;
+        case DWG_TYPE_BLOCK_r11:
+          dwg_free_BLOCK (dat, obj);
+          break;
+        case DWG_TYPE_INSERT_r11:
+          dwg_free_INSERT (dat, obj);
+          break;
+        case DWG_TYPE_DIMENSION_r11:
+          switch (obj->tio.entity->flag_r11)
+            {
+            case 64:
+              dwg_free_DIMENSION_LINEAR (dat, obj);
+              break;
+            case 65:
+              dwg_free_DIMENSION_ALIGNED (dat, obj);
+              break;
+            case 66:
+              dwg_free_DIMENSION_ANG2LN (dat, obj);
+              break;
+            case 68:
+              dwg_free_DIMENSION_DIAMETER (dat, obj);
+              break;
+            case 72:
+              dwg_free_DIMENSION_RADIUS (dat, obj);
+              break;
+            case 80:
+              dwg_free_DIMENSION_ANG3PT (dat, obj);
+              break;
+            case 96:
+              dwg_free_DIMENSION_ORDINATE (dat, obj);
+              break;
+            default:
+              LOG_ERROR ("Unknown preR11 %s.flag_r11 %d", obj->name,
+                         obj->tio.entity->flag_r11)
+            }
+          break;
+        // now the rest
+        default:
+          dwg_free_POINT (dat, obj);
+          break;
+        }
+    }
+
+  // we could also use (Dwg_Object_Type_r11)obj->type
+  switch (obj->fixedtype)
+    {
+    case DWG_TYPE_TEXT:
+      dwg_free_TEXT (dat, obj);
+      break;
+    case DWG_TYPE_ATTRIB:
+      dwg_free_ATTRIB (dat, obj);
+      break;
+    case DWG_TYPE_ATTDEF:
+      dwg_free_ATTDEF (dat, obj);
+      break;
+    case DWG_TYPE_BLOCK:
+      dwg_free_BLOCK (dat, obj);
+      break;
+    case DWG_TYPE_ENDBLK:
+      dwg_free_ENDBLK (dat, obj);
+      break;
+    case DWG_TYPE_SEQEND:
+      dwg_free_SEQEND (dat, obj);
+      break;
+    case DWG_TYPE_REPEAT:
+      dwg_free_REPEAT (dat, obj);
+      break;
+    case DWG_TYPE_ENDREP:
+      dwg_free_ENDREP (dat, obj);
+      break;
+    case DWG_TYPE_INSERT:
+      dwg_free_INSERT (dat, obj);
+      break;
+    case DWG_TYPE_VERTEX_2D:
+      dwg_free_VERTEX_2D (dat, obj);
+      break;
+    case DWG_TYPE_VERTEX_3D:
+      dwg_free_VERTEX_3D (dat, obj);
+      break;
+    case DWG_TYPE_VERTEX_MESH:
+      dwg_free_VERTEX_MESH (dat, obj);
+      break;
+    case DWG_TYPE_VERTEX_PFACE:
+      dwg_free_VERTEX_PFACE (dat, obj);
+      break;
+    case DWG_TYPE_VERTEX_PFACE_FACE:
+      dwg_free_VERTEX_PFACE_FACE (dat, obj);
+      break;
+    case DWG_TYPE_POLYLINE_2D:
+      dwg_free_POLYLINE_2D (dat, obj);
+      break;
+    case DWG_TYPE_POLYLINE_3D:
+      dwg_free_POLYLINE_3D (dat, obj);
+      break;
+    case DWG_TYPE_POLYLINE_PFACE:
+      dwg_free_POLYLINE_PFACE (dat, obj);
+      break;
+    case DWG_TYPE_POLYLINE_MESH:
+      dwg_free_POLYLINE_MESH (dat, obj);
+      break;
+    case DWG_TYPE_ARC:
+      dwg_free_ARC (dat, obj);
+      break;
+    case DWG_TYPE_CIRCLE:
+      dwg_free_CIRCLE (dat, obj);
+      break;
+    case DWG_TYPE_LINE:
+      dwg_free_LINE (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_ORDINATE:
+      dwg_free_DIMENSION_ORDINATE (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_LINEAR:
+      dwg_free_DIMENSION_LINEAR (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_ALIGNED:
+      dwg_free_DIMENSION_ALIGNED (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_ANG3PT:
+      dwg_free_DIMENSION_ANG3PT (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_ANG2LN:
+      dwg_free_DIMENSION_ANG2LN (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_RADIUS:
+      dwg_free_DIMENSION_RADIUS (dat, obj);
+      break;
+    case DWG_TYPE_DIMENSION_DIAMETER:
+      dwg_free_DIMENSION_DIAMETER (dat, obj);
+      break;
+    case DWG_TYPE_POINT:
+      dwg_free_POINT (dat, obj);
+      break;
+    case DWG_TYPE__3DFACE:
+      dwg_free__3DFACE (dat, obj);
+      break;
+    case DWG_TYPE__3DLINE:
+      dwg_free__3DLINE (dat, obj);
+      break;
+    case DWG_TYPE_SOLID:
+      dwg_free_SOLID (dat, obj);
+      break;
+    case DWG_TYPE_TRACE:
+      dwg_free_TRACE (dat, obj);
+      break;
+    case DWG_TYPE_SHAPE:
+      dwg_free_SHAPE (dat, obj);
+      break;
+    case DWG_TYPE_VIEWPORT:
+      dwg_free_VIEWPORT (dat, obj);
+      break;
+    case DWG_TYPE_BLOCK_CONTROL:
+      // _ctrl->entries
+      dwg_free_BLOCK_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_BLOCK_HEADER:
+      dwg_free_BLOCK_HEADER (dat, obj);
+      break;
+    case DWG_TYPE_LAYER_CONTROL:
+      dwg_free_LAYER_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_LAYER:
+      dwg_free_LAYER (dat, obj);
+      break;
+    case DWG_TYPE_STYLE_CONTROL:
+      dwg_free_STYLE_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_STYLE:
+      dwg_free_STYLE (dat, obj);
+      break;
+    case DWG_TYPE_LTYPE_CONTROL:
+      dwg_free_LTYPE_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_LTYPE:
+      dwg_free_LTYPE (dat, obj);
+      break;
+    case DWG_TYPE_VIEW_CONTROL:
+      dwg_free_VIEW_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_VIEW:
+      dwg_free_VIEW (dat, obj);
+      break;
+    case DWG_TYPE_UCS_CONTROL:
+      dwg_free_UCS_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_UCS:
+      dwg_free_UCS (dat, obj);
+      break;
+    case DWG_TYPE_VPORT_CONTROL:
+      dwg_free_VPORT_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_VPORT:
+      dwg_free_VPORT (dat, obj);
+      break;
+    case DWG_TYPE_APPID_CONTROL:
+      dwg_free_APPID_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_APPID:
+      dwg_free_APPID (dat, obj);
+      break;
+    case DWG_TYPE_DIMSTYLE_CONTROL:
+      dwg_free_DIMSTYLE_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_DIMSTYLE:
+      dwg_free_DIMSTYLE (dat, obj);
+      break;
+    case DWG_TYPE_VX_CONTROL:
+      dwg_free_VX_CONTROL (dat, obj);
+      break;
+    case DWG_TYPE_VX_TABLE_RECORD:
+      dwg_free_VX_TABLE_RECORD (dat, obj);
+      break;
+    case DWG_TYPE_LOAD:
+      dwg_free_LOAD (dat, obj);
+      break;
+    case DWG_TYPE_JUMP:
+      dwg_free_JUMP (dat, obj);
+      break;
+    case DWG_TYPE_DICTIONARY:
+      dwg_free_DICTIONARY (dat, obj);
+      break;
+    case DWG_TYPE_UNUSED:
+      // deleted entity. leak? see above
+      break;
+    default:
+      LOG_ERROR (
+          "Unhandled preR13 class %s, fixedtype %d in free_preR13_object()",
+          dwg_type_name (obj->fixedtype), (int)obj->fixedtype);
+    }
+
+  /* With indxf and injson the dxfname is dynamic, just the name is const */
+  if (dwg->opts & DWG_OPTS_IN)
+    FREE_IF (obj->dxfname);
+  /* With injson even the name is dynamic */
+  if (dwg->opts & DWG_OPTS_INJSON)
+    FREE_IF (obj->name);
+  obj->type = DWG_TYPE_FREED;
+}
+
 // using the global dat
 EXPORT void
 dwg_free_object (Dwg_Object *obj)
@@ -609,6 +965,16 @@ dwg_free_object (Dwg_Object *obj)
   if (obj->type == DWG_TYPE_FREED || obj->tio.object == NULL)
     return;
 
+  PRE (R_13b1)
+  {
+    free_preR13_object (obj);
+    return;
+  }
+  if (obj->fixedtype == DWG_TYPE_TABLESTYLE
+      && dwg->header.from_version > R_2007)
+    {
+      free_TABLESTYLE_r2010 (dat, obj);
+    }
   switch (obj->type)
     {
     case DWG_TYPE_TEXT:
@@ -854,7 +1220,7 @@ dwg_free_object (Dwg_Object *obj)
       if (obj->type == obj->parent->layout_type
           && obj->fixedtype == DWG_TYPE_LAYOUT)
         {
-          SINCE (R_13)
+          SINCE (R_13b1)
           {
             dwg_free_LAYOUT (dat, obj); // XXX avoid double-free, esp. in eed
           }
@@ -863,13 +1229,13 @@ dwg_free_object (Dwg_Object *obj)
                & DWG_ERR_UNHANDLEDCLASS)
         {
           if (obj->fixedtype == DWG_TYPE_UNKNOWN_ENT)
-              dwg_free_UNKNOWN_ENT (dat, obj);
+            dwg_free_UNKNOWN_ENT (dat, obj);
           else if (obj->fixedtype == DWG_TYPE_UNKNOWN_OBJ)
-              dwg_free_UNKNOWN_OBJ (dat, obj);
+            dwg_free_UNKNOWN_OBJ (dat, obj);
         }
     }
   /* With indxf the dxfname is dynamic, just the name is const */
-  if (dwg->opts & DWG_OPTS_IN)
+  if ((dwg->opts & DWG_OPTS_INDXF) || (dwg->opts & DWG_OPTS_INJSON))
     FREE_IF (obj->dxfname);
   /* With injson even the name is dynamic */
   if (dwg->opts & DWG_OPTS_INJSON)
@@ -1144,9 +1510,10 @@ dwg_free_object_private (Dwg_Object *obj)
       if (obj->type == obj->parent->layout_type
           && obj->fixedtype == DWG_TYPE_LAYOUT)
         {
-          SINCE (R_13)
+          SINCE (R_13b1)
           {
-            dwg_free_LAYOUT_private (dat, dat, dat, obj); // XXX avoid double-free, esp. in eed
+            dwg_free_LAYOUT_private (
+                dat, dat, dat, obj); // XXX avoid double-free, esp. in eed
           }
         }
       else
@@ -1154,6 +1521,24 @@ dwg_free_object_private (Dwg_Object *obj)
     }
 }
 
+static int
+dwg_free_preR13_header_vars (Dwg_Data *dwg)
+{
+  Dwg_Header_Variables *_obj = &dwg->header_vars;
+  Dwg_Object *obj = NULL;
+  Bit_Chain *dat = &pdat;
+  int error = 0;
+
+  // fields added by dwg_add_Document:
+  FIELD_TV (MENU, 0);
+
+  // clang-format off
+  #include "header_variables_r11.spec"
+  // clang-format on
+
+  return 0;
+}
+
 static int
 dwg_free_header_vars (Dwg_Data *dwg)
 {
@@ -1165,6 +1550,7 @@ dwg_free_header_vars (Dwg_Data *dwg)
   #include "header_variables.spec"
   // clang-format on
 
+  FIELD_TV (DWGCODEPAGE, 0);
   return 0;
 }
 
@@ -1175,7 +1561,7 @@ dwg_free_summaryinfo (Dwg_Data *dwg)
   Dwg_Object *obj = NULL;
   Bit_Chain *dat = &pdat;
 
-  // clang-format off
+// clang-format off
   #include "summaryinfo.spec"
   // clang-format on
   return 0;
@@ -1188,7 +1574,7 @@ dwg_free_appinfo (Dwg_Data *dwg)
   Dwg_Object *obj = NULL;
   Bit_Chain *dat = &pdat;
 
-  // clang-format off
+// clang-format off
   #include "appinfo.spec"
   // clang-format on
   return 0;
@@ -1201,7 +1587,7 @@ dwg_free_filedeplist (Dwg_Data *dwg)
   Bit_Chain *dat = &pdat;
   BITCODE_RL vcount;
 
-  // clang-format off
+// clang-format off
   #include "filedeplist.spec"
   // clang-format on
   return 0;
@@ -1213,7 +1599,7 @@ dwg_free_security (Dwg_Data *dwg)
   Dwg_Object *obj = NULL;
   Bit_Chain *dat = &pdat;
 
-  // clang-format off
+// clang-format off
   #include "security.spec"
   // clang-format on
   return 0;
@@ -1228,7 +1614,7 @@ dwg_free_acds (Dwg_Data *dwg)
   BITCODE_RL rcount3 = 0, rcount4, vcount;
   int error = 0;
 
-  // clang-format off
+// clang-format off
   #include "acds.spec"
   // clang-format on
   return 0;
@@ -1264,9 +1650,11 @@ dwg_free (Dwg_Data *dwg)
           if (!dwg_obj_is_control (&dwg->object[i]))
             dwg_free_object (&dwg->object[i]);
         }
-      dwg_free_header_vars (dwg);
+      if (dwg->header.version < R_13b1)
+        dwg_free_preR13_header_vars (dwg);
+      else
+        dwg_free_header_vars (dwg);
       dwg_free_summaryinfo (dwg);
-      FREE_IF (dwg->thumbnail.chain);
       if (dwg->header.section_infohdr.num_desc)
         {
           for (i = 0; i < dwg->header.section_infohdr.num_desc; ++i)
@@ -1278,15 +1666,15 @@ dwg_free (Dwg_Data *dwg)
       dwg_free_security (dwg);
       dwg_free_acds (dwg);
 
+      FREE_IF (dwg->thumbnail.chain);
       FREE_IF (dwg->vbaproject.unknown_bits);
       FREE_IF (dwg->revhistory.histories);
       FREE_IF (dwg->appinfohistory.unknown_bits);
-      //FREE_IF (dwg->objfreespace...);
+      // FREE_IF (dwg->objfreespace...);
       FREE_IF (dwg->Template.description);
       FREE_IF (dwg->header.section);
-      for (i = 0; i < dwg->second_header.num_handlers; i++)
-        FREE_IF (dwg->second_header.handlers[i].data);
-      // auxheader has no strings
+      FREE_IF (dwg->auxheader.R11_HANDSEED);
+
       for (i = 0; i < dwg->num_objects; ++i)
         {
           if (dwg_obj_is_control (&dwg->object[i]))
@@ -1309,7 +1697,7 @@ dwg_free (Dwg_Data *dwg)
           LOG_HANDLE ("free %d global refs\n", dwg->num_object_refs)
           for (i = 0; i < dwg->num_object_refs; ++i)
             {
-              //LOG_HANDLE ("free ref %d\n", i)
+              LOG_INSANE ("free ref %d\n", i)
               FREE_IF (dwg->object_ref[i]);
             }
         }
@@ -1321,7 +1709,10 @@ dwg_free (Dwg_Data *dwg)
       FREE_IF (dwg->acis_sab_hdl);
       FREE_IF (dwg->object);
       if (dwg->object_map)
-        hash_free (dwg->object_map);
+        {
+          hash_free (dwg->object_map);
+          dwg->object_map = NULL;
+        }
       dwg->num_objects = dwg->num_classes = dwg->num_object_refs = 0;
 #undef FREE_IF
     }
