@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2023 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -14,14 +14,14 @@
  * out_geojson.c: write as GeoJSON
  * written by Reini Urban
  */
-/* TODO: Arc, Circle, Ellipsis, Bulge (Curve) tessellation.
- *       ocs/ucs transforms, explode of inserts?
+/* FIXME: Arc, Circle, Ellipsis, Bulge (Curve) arc_split.
+ * TODO: ocs/ucs transforms, explode of inserts?
  *       NOCOMMA:
  *         We really have to add the comma before, not after, and special case
  *         the first field, not the last to omit the comma.
- *       GeoJSON 2008 or newer RFC7946 https://tools.ietf.org/html/rfc7946#appendix-B
- *       For the new format we need to follow the right-hand rule for orientation
- *       (counterclockwise polygons).
+ *       GeoJSON 2008 or newer RFC7946
+ * https://tools.ietf.org/html/rfc7946#appendix-B For the new format we need to
+ * follow the right-hand rule for orientation (counterclockwise polygons).
  */
 
 #include "config.h"
@@ -33,24 +33,31 @@
 #include <assert.h>
 
 #define IS_PRINT
-#include "common.h"
-#include "bits.h"
-#include "myalloca.h"
 #include "dwg.h"
-#include "decode.h"
-#include "out_json.h"
-
 #define DWG_LOGLEVEL DWG_LOGLEVEL_NONE
 #include "logging.h"
 #include "dwg_api.h"
 
-/* the current version per spec block */
-static unsigned int cur_ver = 0;
+#include "common.h"
+#include "bits.h"
+#include "dwg.h"
+#include "decode.h"
+#include "out_json.h"
+#include "geom.h"
 
-/* https://tools.ietf.org/html/rfc7946#section-11.2 */
-// The default %f is good enough
-//#undef FORMAT_RD
-//#define FORMAT_RD %.6f
+/* the current version per spec block */
+// static unsigned int cur_ver = 0;
+
+/* https://tools.ietf.org/html/rfc7946#section-11.2 recommends.
+   Set via --with-geojson-precision=rfc */
+#undef FORMAT_RD
+#ifndef GEOJSON_PRECISION
+#  define GEOJSON_PRECISION 6
+#endif
+#define FORMAT_RD "%0." _XSTR (GEOJSON_PRECISION) "f"
+//#define FORMAT_RD "%f"
+#undef FORMAT_BD
+#define FORMAT_BD FORMAT_RD
 
 /*--------------------------------------------------------------------------------
  * See http://geojson.org/geojson-spec.html
@@ -136,35 +143,44 @@ static unsigned int cur_ver = 0;
   }
 #define ENDSEC() ENDARRAY
 #define OLD_NOCOMMA fseek (dat->fh, -2, SEEK_CUR)
-#define NOCOMMA assert(0 = "NOCOMMA")
-#define PAIR_S(name, str)                                                     \
+#define NOCOMMA assert (0 = "NOCOMMA")
+// guaranteed non-null str
+#define PAIR_Sc(name, str)                                                    \
   {                                                                           \
-    const int len = strlen (str);                                             \
-    if (len < 4096 / 6)                                                       \
+    const size_t len = strlen (str);                                          \
+    if (len < 42)                                                             \
       {                                                                       \
-        const int _len = 6 * len + 1;                                         \
-        char *_buf = (char *)alloca (_len);                                   \
+        const size_t _len = 6 * len + 1;                                      \
+        char _buf[256];                                                       \
         PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\",\n",                  \
-                        json_cquote (_buf, str, _len));                       \
-        freea (_buf);                                                         \
+                        json_cquote (_buf, str, _len, dat->codepage));        \
       }                                                                       \
     else                                                                      \
       {                                                                       \
-        const int _len = 6 * len + 1;                                         \
+        const size_t _len = 6 * len + 1;                                      \
         char *_buf = (char *)malloc (_len);                                   \
         PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\",\n",                  \
-                        json_cquote (_buf, str, _len));                       \
+                        json_cquote (_buf, str, _len, dat->codepage));        \
         free (_buf);                                                          \
       }                                                                       \
   }
+#define PAIR_S(name, str)                                                     \
+  if (str)                                                                    \
+  PAIR_Sc (name, str)
 #define PAIR_D(name, value)                                                   \
   {                                                                           \
     PREFIX fprintf (dat->fh, "\"" #name "\": %d,\n", value);                  \
   }
-#define LASTPAIR_S(name, value)                                               \
+// guaranteed non-null str
+#define LASTPAIR_Sc(name, value)                                              \
   {                                                                           \
     PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\"\n", value);               \
   }
+#define LASTPAIR_S(name, value)                                               \
+  if (value)                                                                  \
+    {                                                                         \
+      PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\"\n", value);             \
+    }
 #define PAIR_NULL(name)                                                       \
   {                                                                           \
     PREFIX fprintf (dat->fh, "\"" #name "\": null,\n");                       \
@@ -185,9 +201,9 @@ static unsigned int cur_ver = 0;
   }
 #define ENDGEOMETRY LASTENDHASH
 
-//#define VALUE(value,type,dxf)
-//    fprintf(dat->fh, FORMAT_##type, value)
-//#define VALUE_RC(value,dxf) VALUE(value, RC, dxf)
+// #define VALUE(value,type,dxf)
+//     fprintf(dat->fh, FORMAT_##type, value)
+// #define VALUE_RC(value,dxf) VALUE(value, RC, dxf)
 
 #define FIELD(name, type, dxf)
 #define _FIELD(name, type, value)
@@ -209,7 +225,7 @@ static unsigned int cur_ver = 0;
 #define FIELD_BS(name, dxf) FIELD (name, BS, dxf)
 #define FIELD_BL(name, dxf) FIELD (name, BL, dxf)
 #define FIELD_BLL(name, dxf) FIELD (name, BLL, dxf)
-#define FIELD_BD(name, dxf)
+#define FIELD_BD(name, dxf) FIELD (name, BD, dxf)
 #define FIELD_RC(name, dxf) FIELD (name, RC, dxf)
 #define FIELD_RS(name, dxf) FIELD (name, RS, dxf)
 #define FIELD_RD(name, dxf) FIELD_BD (name, dxf)
@@ -240,12 +256,12 @@ static unsigned int cur_ver = 0;
 #define _VALUE_RD(value) fprintf (dat->fh, FORMAT_RD, value)
 #ifdef IS_RELEASE
 #  define VALUE_RD(value)                                                     \
-  {                                                                           \
-    if (bit_isnan (value))                                                    \
-      _VALUE_RD (0.0);                                                        \
-    else                                                                      \
-      _VALUE_RD (value);                                                      \
-   }
+    {                                                                         \
+      if (bit_isnan (value))                                                  \
+        _VALUE_RD (0.0);                                                      \
+      else                                                                    \
+        _VALUE_RD (value);                                                    \
+    }
 #else
 #  define VALUE_RD(value) _VALUE_RD (value)
 #endif
@@ -267,7 +283,7 @@ static unsigned int cur_ver = 0;
   }
 #define FIELD_2DPOINT(name) VALUE_2DPOINT (_obj->name.x, _obj->name.y)
 #define LASTFIELD_2DPOINT(name) LASTVALUE_2DPOINT (_obj->name.x, _obj->name.y)
-#define VALUE_3DPOINT(px, py, pz)                                          \
+#define VALUE_3DPOINT(px, py, pz)                                             \
   {                                                                           \
     PREFIX fprintf (dat->fh, "[ ");                                           \
     VALUE_RD (px);                                                            \
@@ -298,7 +314,7 @@ static unsigned int cur_ver = 0;
     if (_obj->name.z != 0.0)                                                  \
       VALUE_3DPOINT (_obj->name.x, _obj->name.y, _obj->name.z)                \
     else                                                                      \
-      FIELD_2DPOINT(name)                                                     \
+      FIELD_2DPOINT (name)                                                    \
   }
 #define LASTFIELD_3DPOINT(name)                                               \
   {                                                                           \
@@ -327,19 +343,19 @@ static unsigned int cur_ver = 0;
 #define FIELD_VECTOR_T(name, type, size, dxf)                                 \
   ARRAY;                                                                      \
   if (!(IS_FROM_TU (dat)))                                                    \
-  {                                                                           \
-    for (vcount = 0; vcount < (BITCODE_BL)_obj->size; vcount++)               \
-      {                                                                       \
-        PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\"%s\n",                 \
-                        _obj->name[vcount],                                   \
-                        vcount == (BITCODE_BL)_obj->size - 1 ? "" : ",");     \
-      }                                                                       \
-  }                                                                           \
+    {                                                                         \
+      for (vcount = 0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
+        {                                                                     \
+          PREFIX fprintf (dat->fh, "\"" #name "\": \"%s\"%s\n",               \
+                          _obj->name[vcount],                                 \
+                          vcount == (BITCODE_BL)_obj->size - 1 ? "" : ",");   \
+        }                                                                     \
+    }                                                                         \
   else                                                                        \
-  {                                                                           \
-    for (vcount = 0; vcount < (BITCODE_BL)_obj->size; vcount++)               \
-      FIELD_TEXT_TU (name, _obj->name[vcount]);                               \
-  }                                                                           \
+    {                                                                         \
+      for (vcount = 0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
+        FIELD_TEXT_TU (name, _obj->name[vcount]);                             \
+    }                                                                         \
   ENDARRAY;
 
 #define FIELD_VECTOR(name, type, size, dxf)                                   \
@@ -360,9 +376,9 @@ static unsigned int cur_ver = 0;
   ENDARRAY;
 
 #define WARN_UNSTABLE_CLASS                                                   \
-  LOG_WARN ("Unstable Class %s %d %s (0x%x%s) -@%ld",                         \
+  LOG_WARN ("Unstable Class %s %d %s (0x%x%s) -@%" PRIuSIZE,                  \
             is_entity ? "entity" : "object", klass->number, dxfname,          \
-            klass->proxyflag, klass->is_zombie ? "is_zombie" : "",          \
+            klass->proxyflag, klass->is_zombie ? "is_zombie" : "",            \
             obj->address + obj->size)
 
 // ensure counter-clockwise orientation of a closed polygon. 2d only.
@@ -374,25 +390,26 @@ normalize_polygon_orient (BITCODE_BL numpts, dwg_point_2d **const pts_p)
   // check orientation
   for (unsigned i = 0; i < numpts - 1; i++)
     {
-      sum += (pts[i+1].x - pts[i].x) * (pts[i+1].y + pts[i].y);
+      sum += (pts[i + 1].x - pts[i].x) * (pts[i + 1].y + pts[i].y);
     }
   if (sum > 0.0) // if clockwise
     {
       // reverse and return a copy
       unsigned last = numpts - 1;
-      dwg_point_2d *new = malloc (numpts * sizeof (BITCODE_2RD));
-      //fprintf (stderr, "%u pts, sum %f: reverse orient\n", numpts, sum);
+      dwg_point_2d *newpts
+          = (dwg_point_2d *)malloc (numpts * sizeof (BITCODE_2RD));
+      // fprintf (stderr, "%u pts, sum %f: reverse orient\n", numpts, sum);
       for (unsigned i = 0; i < numpts; i++)
         {
-          new[i].x = pts[last-i].x;
-          new[i].y = pts[last-i].y;
+          newpts[i].x = pts[last - i].x;
+          newpts[i].y = pts[last - i].y;
         }
-      *pts_p = new;
+      *pts_p = newpts;
       return 1;
     }
   else
     {
-      //fprintf (stderr, "%u pts, sum %f: keep orient\n", numpts, sum);
+      // fprintf (stderr, "%u pts, sum %f: keep orient\n", numpts, sum);
       return 0;
     }
 }
@@ -406,9 +423,9 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
   char *name;
   char tmp[64];
 
-  PAIR_S (type, "Feature");
-  sprintf (tmp, "%lX", obj->handle.value);
-  PAIR_S (id, tmp);
+  PAIR_Sc (type, "Feature");
+  snprintf (tmp, sizeof (tmp), FORMAT_RLLx, obj->handle.value);
+  PAIR_Sc (id, tmp);
   KEY (properties);
   SAMEHASH;
   PAIR_S (SubClasses, subclass);
@@ -417,8 +434,8 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
       Dwg_Object *layer
           = obj->tio.entity->layer ? obj->tio.entity->layer->obj : NULL;
       if (layer
-          && (layer->type == DWG_TYPE_LAYER
-              || layer->type == DWG_TYPE_DICTIONARY))
+          && (layer->fixedtype == DWG_TYPE_LAYER
+              || layer->fixedtype == DWG_TYPE_DICTIONARY))
         {
           name = dwg_obj_table_get_name (layer, &error);
           if (!error)
@@ -435,8 +452,9 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
               || obj->tio.entity->color.method == 0xc2) // Entity
           && obj->tio.entity->color.index == 256)
         {
-          sprintf (tmp, "#%06X", obj->tio.entity->color.rgb & 0xffffff);
-          PAIR_S (Color, tmp);
+          snprintf (tmp, sizeof (tmp), "#%06X",
+                    obj->tio.entity->color.rgb & 0xffffff);
+          PAIR_Sc (Color, tmp);
         }
       else if ((obj->tio.entity->color.index != 256)
                || (dat->version >= R_2004
@@ -459,7 +477,7 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
     }
 
   // if has notes and opt. an mtext frame_text
-  if (obj->type == DWG_TYPE_GEOPOSITIONMARKER)
+  if (obj->fixedtype == DWG_TYPE_GEOPOSITIONMARKER)
     {
       Dwg_Entity_GEOPOSITIONMARKER *_obj
           = obj->tio.entity->tio.GEOPOSITIONMARKER;
@@ -474,7 +492,7 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
           PAIR_S (Text, _obj->notes)
         }
     }
-  else if (obj->type == DWG_TYPE_TEXT)
+  else if (obj->fixedtype == DWG_TYPE_TEXT)
     {
       Dwg_Entity_TEXT *_obj = obj->tio.entity->tio.TEXT;
       if (IS_FROM_TU (dat))
@@ -482,13 +500,13 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
           char *utf8 = bit_convert_TU ((BITCODE_TU)_obj->text_value);
           PAIR_S (Text, utf8)
           free (utf8);
-        } 
+        }
       else
-        {      
+        {
           PAIR_S (Text, _obj->text_value)
         }
     }
-  else if (obj->type == DWG_TYPE_MTEXT)
+  else if (obj->fixedtype == DWG_TYPE_MTEXT)
     {
       Dwg_Entity_MTEXT *_obj = obj->tio.entity->tio.MTEXT;
       if (IS_FROM_TU (dat))
@@ -496,17 +514,17 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
           char *utf8 = bit_convert_TU ((BITCODE_TU)_obj->text);
           PAIR_S (Text, utf8)
           free (utf8);
-        } 
+        }
       else
-        {      
+        {
           PAIR_S (Text, _obj->text)
         }
     }
-  else if (obj->type == DWG_TYPE_INSERT)
+  else if (obj->fixedtype == DWG_TYPE_INSERT)
     {
       Dwg_Entity_INSERT *_obj = obj->tio.entity->tio.INSERT;
       Dwg_Object *hdr = dwg_ref_get_object (_obj->block_header, &error);
-      if (!error && hdr && hdr->type == DWG_TYPE_BLOCK_HEADER)
+      if (!error && hdr && hdr->fixedtype == DWG_TYPE_BLOCK_HEADER)
         {
           Dwg_Object_BLOCK_HEADER *_hdr = hdr->tio.object->tio.BLOCK_HEADER;
           char *text;
@@ -522,11 +540,11 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
             }
         }
     }
-  else if (obj->type == DWG_TYPE_MINSERT)
+  else if (obj->fixedtype == DWG_TYPE_MINSERT)
     {
       Dwg_Entity_MINSERT *_obj = obj->tio.entity->tio.MINSERT;
       Dwg_Object *hdr = dwg_ref_get_object (_obj->block_header, &error);
-      if (!error && hdr && hdr->type == DWG_TYPE_BLOCK_HEADER)
+      if (!error && hdr && hdr->fixedtype == DWG_TYPE_BLOCK_HEADER)
         {
           Dwg_Object_BLOCK_HEADER *_hdr = hdr->tio.object->tio.BLOCK_HEADER;
           char *text;
@@ -543,8 +561,8 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
         }
     }
   // PAIR_NULL(ExtendedEntity);
-  sprintf (tmp, "%lX", obj->handle.value);
-  LASTPAIR_S (EntityHandle, tmp);
+  snprintf (tmp, sizeof (tmp), FORMAT_RLLx, obj->handle.value);
+  LASTPAIR_Sc (EntityHandle, tmp);
   ENDHASH;
 }
 
@@ -558,17 +576,23 @@ dwg_geojson_feature (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
     ENDHASH
 
 static int
-dwg_geojson_LWPOLYLINE (Bit_Chain *restrict dat, Dwg_Object *restrict obj, int is_last)
+dwg_geojson_LWPOLYLINE (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
+                        int is_last)
 {
   BITCODE_BL j, last_j;
   Dwg_Entity_LWPOLYLINE *_obj = obj->tio.entity->tio.LWPOLYLINE;
-  dwg_point_2d *pts = (dwg_point_2d*)_obj->points;
+  dwg_point_2d *pts = (dwg_point_2d *)_obj->points;
+  if (!_obj->points)
+    return 1;
 
   FEATURE (AcDbEntity : AcDbLwPolyline, obj);
+  // TODO bulges, splines, ...
+
   // if closed and num_points > 3 use a Polygon
   if (_obj->flag & 512 && _obj->num_points > 3)
     {
-      int changed = normalize_polygon_orient (_obj->num_points, &pts); // RFC7946
+      int changed
+          = normalize_polygon_orient (_obj->num_points, &pts); // RFC7946
       GEOMETRY (Polygon)
       KEY (coordinates);
       ARRAY;
@@ -608,9 +632,12 @@ dwg_geojson_variable_type (Dwg_Data *restrict dwg, Bit_Chain *restrict dat,
   Dwg_Class *klass;
   int is_entity;
 
-  i = obj->type - 500;
+  i = obj->fixedtype - 500;
   if (i < 0 || i >= (int)dwg->num_classes)
     return 0;
+  if (obj->fixedtype == DWG_TYPE_UNKNOWN_ENT
+      || obj->fixedtype == DWG_TYPE_UNKNOWN_OBJ)
+    return DWG_ERR_UNHANDLEDCLASS;
 
   klass = &dwg->dwg_class[i];
   if (!klass || !klass->dxfname)
@@ -657,9 +684,10 @@ dwg_geojson_variable_type (Dwg_Data *restrict dwg, Bit_Chain *restrict dat,
 }
 
 static int
-dwg_geojson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj, int is_last)
+dwg_geojson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
+                    int is_last)
 {
-  switch (obj->type)
+  switch (obj->fixedtype)
     {
     case DWG_TYPE_INSERT: // Just the insertion point yet
       {
@@ -682,15 +710,21 @@ dwg_geojson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj, int is_la
       {
         int error;
         BITCODE_BL j, numpts;
-        bool is_polygon = false;
+        // bool is_polygon = false;
         int changed = 0;
         dwg_point_2d *pts, *orig;
         Dwg_Entity_POLYLINE_2D *_obj = obj->tio.entity->tio.POLYLINE_2D;
+        numpts = dwg_object_polyline_2d_get_numpoints (obj, &error);
+        if (error || !numpts)
+          return 0;
+        pts = dwg_object_polyline_2d_get_points (obj, &error);
+        if (error || !pts)
+          return 0;
+        // TODO bulges needs explosion into lines. divided by polyline curve
+        // smoothness (default 8)
 
         // if closed and num_points > 3 use a Polygon
         FEATURE (AcDbEntity : AcDbPolyline, obj);
-        numpts = dwg_object_polyline_2d_get_numpoints (obj, &error);
-        pts = dwg_object_polyline_2d_get_points (obj, &error);
         if (_obj->flag & 512 && numpts > 3)
           {
             orig = pts; // pts is already a new copy
@@ -733,12 +767,16 @@ dwg_geojson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj, int is_la
         int error;
         BITCODE_BL j, numpts;
         dwg_point_3d *pts;
+        numpts = dwg_object_polyline_3d_get_numpoints (obj, &error);
+        if (error || !numpts)
+          return 0;
+        pts = dwg_object_polyline_3d_get_points (obj, &error);
+        if (error || !pts)
+          return 0;
         FEATURE (AcDbEntity : AcDbPolyline, obj);
         GEOMETRY (LineString);
         KEY (coordinates);
         ARRAY;
-        numpts = dwg_object_polyline_3d_get_numpoints (obj, &error);
-        pts = dwg_object_polyline_3d_get_points (obj, &error);
         for (j = 0; j < numpts; j++)
           {
             if (j == numpts - 1)
@@ -757,14 +795,84 @@ dwg_geojson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj, int is_la
         return 1;
       }
     case DWG_TYPE_ARC:
-      // needs explosion of arc into lines
       // dwg_geojson_ARC(dat, obj);
-      LOG_TRACE ("ARC not yet supported")
+      if (1)
+        {
+          Dwg_Entity_ARC *_obj = obj->tio.entity->tio.ARC;
+          const int viewres = 1000;
+          BITCODE_2BD ctr = { _obj->center.x, _obj->center.y };
+          BITCODE_2BD *pts;
+          int num_pts;
+          double end_angle = _obj->end_angle;
+          // viewres is for 2PI. we need anglediff(deg)/2PI
+          while (end_angle - _obj->start_angle < 1e-6)
+            end_angle += M_PI;
+          num_pts
+              = (int)trunc (viewres / rad2deg (end_angle - _obj->start_angle));
+          if (num_pts > 10000 || num_pts < 0)
+            {
+              LOG_ERROR ("Invalid angles");
+              return DWG_ERR_VALUEOUTOFBOUNDS;
+            }
+          num_pts = MIN (num_pts, 120);
+          pts = (BITCODE_2BD *)malloc (num_pts * sizeof (BITCODE_2BD));
+          if (!pts)
+            {
+              LOG_ERROR ("Out of memory");
+              return DWG_ERR_OUTOFMEM;
+            }
+          // explode into line segments. divided by VIEWRES (default 1000)
+          arc_split (pts, num_pts, ctr, _obj->start_angle, _obj->end_angle,
+                     _obj->radius);
+          FEATURE (AcDbEntity : AcDbArc, obj);
+          GEOMETRY (Polygon)
+          KEY (coordinates);
+          ARRAY;
+          ARRAY;
+          for (int j = 0; j < num_pts; j++)
+            {
+              VALUE_2DPOINT (pts[j].x, pts[j].y)
+            }
+          LASTVALUE_2DPOINT (pts[0].x, pts[0].y);
+          LASTENDARRAY;
+          LASTENDARRAY;
+          ENDGEOMETRY;
+          ENDFEATURE;
+          free (pts);
+        }
+      else
+        LOG_TRACE ("ARC not yet supported")
       break;
     case DWG_TYPE_CIRCLE:
-      // needs explosion of arc into lines
       // dwg_geojson_CIRCLE(dat, obj);
-      LOG_TRACE ("CIRCLE not yet supported")
+      if (1)
+        {
+          Dwg_Entity_CIRCLE *_obj = obj->tio.entity->tio.CIRCLE;
+          // const int viewres = 1000; //dwg->header_vars.VIEWRES;
+          BITCODE_2BD ctr = { _obj->center.x, _obj->center.y };
+          // double res = viewres / 360.0;
+          int num_pts = 120;
+          BITCODE_2BD *pts
+              = (BITCODE_2BD *)malloc (num_pts * sizeof (BITCODE_2BD));
+          arc_split (pts, num_pts, ctr, 0, M_PI * 2.0, _obj->radius);
+          FEATURE (AcDbEntity : AcDbCircle, obj);
+          GEOMETRY (Polygon)
+          KEY (coordinates);
+          ARRAY;
+          ARRAY;
+          for (int j = 0; j < num_pts; j++)
+            {
+              VALUE_2DPOINT (pts[j].x, pts[j].y)
+            }
+          LASTVALUE_2DPOINT (pts[0].x, pts[0].y);
+          LASTENDARRAY;
+          LASTENDARRAY;
+          ENDGEOMETRY;
+          ENDFEATURE;
+          free (pts);
+        }
+      else
+        LOG_TRACE ("CIRCLE not yet supported")
       break;
     case DWG_TYPE_LINE:
       {
@@ -878,7 +986,8 @@ dwg_geojson_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj, int is_la
     case DWG_TYPE_LWPOLYLINE:
       return dwg_geojson_LWPOLYLINE (dat, obj, is_last);
     default:
-      if (obj->type != obj->parent->layout_type)
+      if (obj->parent && dat->version > R_12
+          && obj->fixedtype != obj->parent->layout_type)
         return dwg_geojson_variable_type (obj->parent, dat, obj, is_last);
     }
   return 0;
@@ -895,10 +1004,10 @@ geojson_entities_write (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
       int is_last = i == dwg->num_objects - 1;
       Dwg_Object *obj = &dwg->object[i];
       success = dwg_geojson_object (dat, obj, is_last);
-      if (is_last && !success) // needed for the LASTFEATURE comma. end with an empty dummy
+      if (is_last && !success) // needed for the LASTFEATURE comma. end with an
+                               // empty dummy
         {
-          HASH
-          PAIR_S (type, "Feature");
+          HASH PAIR_Sc (type, "Feature");
           PAIR_NULL (properties);
           LASTPAIR_NULL (geometry);
           LASTENDHASH;
@@ -915,11 +1024,11 @@ dwg_write_geojson (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
   char date[12] = "YYYY-MM-DD";
   time_t rawtime;
 
-  if (!dwg->num_objects)
+  if (!dwg->num_objects || !dat->fh)
     goto fail;
 
   HASH;
-  PAIR_S (type, "FeatureCollection");
+  PAIR_Sc (type, "FeatureCollection");
 
   // array of features
   if (geojson_entities_write (dat, dwg))
@@ -929,15 +1038,15 @@ dwg_write_geojson (Bit_Chain *restrict dat, Dwg_Data *restrict dwg)
   HASH;
   time (&rawtime);
   strftime (date, 12, "%Y-%m-%d", localtime (&rawtime));
-  PAIR_S (creation_date, date);
+  PAIR_Sc (creation_date, date);
   KEY (generator);
   HASH;
   KEY (author);
   HASH;
-  LASTPAIR_S (name, "dwgread");
+  LASTPAIR_Sc (name, "dwgread");
   ENDHASH;
-  PAIR_S (package, PACKAGE_NAME);
-  LASTPAIR_S (version, PACKAGE_VERSION);
+  PAIR_Sc (package, PACKAGE_NAME);
+  LASTPAIR_Sc (version, PACKAGE_VERSION);
   LASTENDHASH;
   // PAIR_S(license, "?");
   LASTENDHASH;
