@@ -41,6 +41,7 @@ static BITCODE_BL rcount1, rcount2;
 
 #define ACTION print
 #define IS_PRINT
+#undef USE_WRITE
 
 #define FIELD(nam, type) FIELD_TRACE (nam, type)
 #define FIELDG(nam, type, dxf) FIELD_G_TRACE (nam, type, dxf)
@@ -52,7 +53,8 @@ static BITCODE_BL rcount1, rcount2;
   LOG_TRACE (#nam ": " FORMAT_##type " [" #type " " #dxf "]\n",               \
              (BITCODE_##type)_obj->nam)
 #define SUB_FIELD(o, nam, type, dxf) FIELDG (o.nam, type, dxf)
-#define SUB_FIELD_CAST(o, nam, type, cast, dxf) FIELD_G_TRACE (o.nam, cast, dxf)
+#define SUB_FIELD_CAST(o, nam, type, cast, dxf)                               \
+  FIELD_G_TRACE (o.nam, cast, dxf)
 
 #define LOG_INSANE_TF(var, len)
 #define FIELD_VALUE(name) _obj->name
@@ -84,7 +86,7 @@ static BITCODE_BL rcount1, rcount2;
   if (handleptr)                                                              \
     {                                                                         \
       LOG_TRACE (#name "[%d]: HANDLE" FORMAT_REF " [%d]\n", (int)vcount,      \
-                 ARGS_REF (handleptr),  dxf);                                 \
+                 ARGS_REF (handleptr), dxf);                                  \
     }
 #define FIELD_HANDLE_N(name, vcount, handle_code, dxf)                        \
   VALUE_HANDLE_N (_obj->name, name, vcount, handle_code, dxf)
@@ -213,6 +215,8 @@ static BITCODE_BL rcount1, rcount2;
 #define VALUE_RC(value, dxf) VALUE (value, RC, dxf)
 #define VALUE_RS(value, dxf) VALUE (value, RS, dxf)
 #define VALUE_RL(value, dxf) VALUE (value, RL, dxf)
+#define VALUE_RLx(value, dxf)                                                 \
+  LOG_TRACE (FORMAT_RLx " [RL " #dxf "]\n", (BITCODE_RL)value)
 #define VALUE_RD(value, dxf) VALUE (value, RD, dxf)
 #define VALUE_BD(value, dxf) VALUE (value, BD, dxf)
 
@@ -233,11 +237,14 @@ static BITCODE_BL rcount1, rcount2;
     {                                                                         \
       for (vcount = 0; vcount < (BITCODE_BL)_obj->size; vcount++)             \
         {                                                                     \
-          PRE (R_2007)                                                        \
+          PRE (R_2007a)                                                        \
           {                                                                   \
             LOG_TRACE (#name "[%ld]: %s\n", (long)vcount, _obj->name[vcount]) \
           }                                                                   \
-          else { LOG_TRACE_TU (#name, _obj->name[vcount], dxf) }              \
+          else                                                                \
+          {                                                                   \
+            LOG_TRACE_TU (#name, _obj->name[vcount], dxf)                     \
+          }                                                                   \
         }                                                                     \
     }
 
@@ -306,9 +313,9 @@ static BITCODE_BL rcount1, rcount2;
     }
 
 #define XDICOBJHANDLE(code)                                                   \
-  SINCE (R_2004)                                                              \
+  SINCE (R_2004a)                                                              \
   {                                                                           \
-    if (!obj->tio.object->is_xdic_missing)                                  \
+    if (!obj->tio.object->is_xdic_missing)                                    \
       VALUE_HANDLE (obj->tio.object->xdicobjhandle, xdicobjhandle, code, 0);  \
   }                                                                           \
   PRIOR_VERSIONS                                                              \
@@ -336,9 +343,10 @@ static BITCODE_BL rcount1, rcount2;
   bit_set_position (hdl_dat, obj->hdlpos)
 
 #define DWG_ENTITY(token)                                                     \
-  static int dwg_print_##token##_private (                                    \
-      Bit_Chain *dat, Bit_Chain *hdl_dat, Bit_Chain *str_dat,                 \
-      const Dwg_Object *restrict obj) {                                       \
+  static int dwg_print_##token##_private (Bit_Chain *dat, Bit_Chain *hdl_dat, \
+                                          Bit_Chain *str_dat,                 \
+                                          const Dwg_Object *restrict obj)     \
+  {                                                                           \
     return 0;                                                                 \
   }                                                                           \
   static int dwg_print_##token (Bit_Chain *restrict dat,                      \
@@ -362,9 +370,10 @@ static BITCODE_BL rcount1, rcount2;
   }
 
 #define DWG_OBJECT(token)                                                     \
-  static int dwg_print_##token##_private (                                    \
-      Bit_Chain *dat, Bit_Chain *hdl_dat, Bit_Chain *str_dat,                 \
-      const Dwg_Object *restrict obj) {                                       \
+  static int dwg_print_##token##_private (Bit_Chain *dat, Bit_Chain *hdl_dat, \
+                                          Bit_Chain *str_dat,                 \
+                                          const Dwg_Object *restrict obj)     \
+  {                                                                           \
     return 0;                                                                 \
   }                                                                           \
   static int dwg_print_##token (Bit_Chain *restrict dat,                      \
@@ -408,7 +417,7 @@ dwg_print_variable_type (Dwg_Data *restrict dwg, Bit_Chain *restrict dat,
   // almost always false
   is_entity = dwg_class_is_entity (klass);
 
-  // clang-format off
+// clang-format off
   #include "classes.inc"
   // clang-format on
 
@@ -422,6 +431,11 @@ int
 dwg_print_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj)
 {
   int error = 0;
+  unsigned int type;
+
+  if (!obj || !obj->parent)
+    return DWG_ERR_INTERNALERROR;
+  type = dat->version < R_13b1 ? (unsigned int)obj->fixedtype : obj->type;
   // Bit_Chain * dat = (Bit_Chain *)obj->parent->bit_chain;
   // Bit_Chain *hdl_dat = dat;
   switch (obj->type)
@@ -587,6 +601,16 @@ dwg_print_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj)
       return dwg_print_PROXY_ENTITY (dat, obj);
     case DWG_TYPE_PROXY_OBJECT: // DXF name: PROXY
       return dwg_print_PROXY_OBJECT (dat, obj);
+    case DWG_TYPE_REPEAT:
+      return dwg_print_REPEAT (dat, obj);
+    case DWG_TYPE_ENDREP:
+      return dwg_print_ENDREP (dat, obj);
+    case DWG_TYPE__3DLINE:
+      return dwg_print__3DLINE (dat, obj);
+    case DWG_TYPE_LOAD:
+      return dwg_print_LOAD (dat, obj);
+    case DWG_TYPE_JUMP:
+      return dwg_print_JUMP (dat, obj);
     default:
       if (obj->type == obj->parent->layout_type)
         {
@@ -618,10 +642,11 @@ dwg_print_object (Bit_Chain *restrict dat, Dwg_Object *restrict obj)
           else // not a class
             {
               LOG_WARN ("Unknown object, skipping eed/reactors/xdic");
-              SINCE (R_2000){
-                LOG_INFO ("Object bitsize: %u\n", obj->bitsize)
-              } LOG_INFO ("Object handle: " FORMAT_H "\n",
-                          ARGS_H (obj->handle));
+              SINCE (R_2000b)
+              {
+                LOG_INFO ("Object bitsize: %u\n", obj->bitsize);
+              }
+              LOG_INFO ("Object handle: " FORMAT_H "\n", ARGS_H (obj->handle));
               return error | DWG_ERR_INVALIDTYPE;
             }
         }
