@@ -4,7 +4,7 @@
 void
 api_process (dwg_object *obj)
 {
-  int error, isnew; 
+  int error, isnew;
   BITCODE_BL class_version;
   BITCODE_BL procedure;
   BITCODE_BL destination;
@@ -16,7 +16,6 @@ api_process (dwg_object *obj)
   BITCODE_B highlevel_info;
 
   Dwg_Version_Type dwg_version = obj->parent->header.version;
-#ifdef DEBUG_CLASSES
   dwg_obj_renderglobal *_obj = dwg_object_to_RENDERGLOBAL (obj);
 
   CHK_ENTITY_TYPE (_obj, RENDERGLOBAL, class_version, BL);
@@ -28,5 +27,4 @@ api_process (dwg_object *obj)
   CHK_ENTITY_TYPE (_obj, RENDERGLOBAL, image_height, BL);
   CHK_ENTITY_TYPE (_obj, RENDERGLOBAL, predef_presets_first, B);
   CHK_ENTITY_TYPE (_obj, RENDERGLOBAL, highlevel_info, B);
-#endif
 }
